%% NUMEXP_TDE_EXAMPLE is an example script for a TDE numerical experiment
% 
% This is an example script used for quick examples and debugging of
% algorithms. It allows for small scale simulations run on a single 
% computer, but similar to those in the experiments folder.
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
clear; clc;
addpath('signal_generators','recon_algorithms','misc');

% Set random seed
seed = 2;
if seed~=0
    s = RandStream('mcg16807','Seed',seed);
    RandStream.setGlobalStream(s);
end

% Experiment variables
kappas = linspace(0.1, 0.5, 5);
measSNRs = 1000;
sigSNRs = 1000;
noRuns = 3;

% Experiment parameters
N = 500;
K = 3;
b_sep = 10^(-6);
eta = 0;
xi = 0;
lambdas = 10^0;

% Signal parameters
min_a = 1;
max_a = 10;

% Function g
f_s = 50*10^6;
dt = 1/f_s;
Delta = dt;
t = (0:(N-1))'*dt;
f_0 = 10^6; delta_f = 40*f_0; T = 10^(-6);
g =@(tau) chirp(t-tau, f_0, delta_f, T);
min_b = 0;
max_b = t(N)-T;

% Dictionary
D = zeros(N);
Omega = (0:Delta:((N-1)*Delta-T));
D(:,1) = g(Omega(1));
for ii = 2:N
    D(:,ii) = circshift(D(:,ii-1),1);
end
mu = abs(D'*D);

% Numerical experiment
setts.N         = N;
setts.K         = K;
setts.Delta     = Delta;
setts.g         = g;
setts.D         = D;
setts.Omega     = Omega;
setts.mu        = mu;
setts.eta       = eta;
setts.xi        = xi;
setts.min_b     = min_b;
setts.max_b     = max_b;
setts.b_sep     = b_sep;
setts.min_a     = min_a;
setts.max_a     = max_a;
setts.dt        = dt;
ii = 1;
methods{ii} = 'BOMP';           ii = ii+1;
methods{ii} = 'PaIBOMP';        ii = ii+1;
methods{ii} = 'TDE-MUSIC';      ii = ii+1;
methods{ii} = 'CCBP';           ii = ii+1;
methods{ii} = 'PaIBOMP+CCBP';   ii = ii+1;
methods{ii} = 'PoIBOMP';        
noMethods = length(methods);

if sum(strcmp(methods, 'CCBP'))
    ccbp_vars = gen_ccbp_variables(setts, Omega);
    setts.ccbp_vars = ccbp_vars;
end

[f_errors_all, bMSE_errors_all, comp_times_all] = stics_numexp(methods, lambdas, kappas, measSNRs, sigSNRs, K, noRuns, setts);
 
% Averaging the results
f_errors = squeeze(mean(f_errors_all, 5));
bMSE_errors = squeeze(mean(bMSE_errors_all, 5));
comp_times = squeeze(mean(comp_times_all, 5));

% Plotting
colors = {  [0 0 0], [0 0 1], [0 1 0], ...
            [0 1 1], [1 0 0], [1 0 1], ...
            [1 1 0], [0.75 0 0.25], [1 1 0.5]};
markers = { 'x', 'o', '+', ...
            'd', '*', 's', ...
            'p', 'h', 'v'};

if length(kappas)==1
    xaxis = measSNRs;
    xaxislabel = 'SNR [dB]';
elseif length(measSNRs)==1
    xaxis = kappas;
    xaxislabel = '\kappa';
end

% f_errors
figure(1); clf;
for ii = 1:noMethods
    semilogy(xaxis, f_errors(ii,:), 'Color', colors{ii}, 'Marker', markers{ii}); hold on
end
xlabel(xaxislabel);
ylabel('Function norm-2 error');

% bMSE
figure(2); clf;
for ii = 1:noMethods
    semilogy(xaxis, bMSE_errors(ii,:), 'Color', colors{ii}, 'Marker', markers{ii}); hold on
end
xlabel(xaxislabel);
ylabel('\b-MSE');

% Computation time
figure(3); clf;
for ii = 1:noMethods
    semilogy(xaxis, comp_times(ii,:), 'Color', colors{ii}, 'Marker', markers{ii}); hold on
end
xlabel(xaxislabel);
ylabel('Average computation time [s]');
legend(methods);