%% APPROXIMATION_ERROR_ANALYSIS finds a bound on the approximation error
% 
% Use this script to find the value of zeta, the reconstruction error due
% to approximation error, that is input to the CCBP algorithm.
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
clear; clc; clf();
addpath('../signal_generators','../misc');

%% Common parameters
no_b_Deltas = 100;
max_c = 30;
c = 1:max_c;

%% TDE parameters
% Experiment parameters
N = 500;
b_i = 10^(-6);

% Function g
f_s = 50*10^6;
dt = 1/f_s;
Deltas = dt * 1./c;
t = (0:(N-1))'*dt;
f_0 = 10^6; delta_f = 40*f_0; T = 10^(-6);
g =@(tau) chirp(t-tau, f_0, delta_f, T);

% Calculate deltas and zetas
[~, zetas_tde, ~] = approximation_error(no_b_Deltas, b_i, Deltas, g);
max_zetas_tde = max(zetas_tde,[],2);

% Calculate regular BOMP errors
max_zetas_bomp_tde = zeros(size(max_zetas_tde));
for ii=1:length(c)
    max_zetas_bomp_tde(ii) = norm(g(b_i)-g(b_i+Deltas(ii)/2),2);
end

%% SCS parameters
% Experiment parameters
N = 100;
Deltas = 1./c;
b_i = 1;

% Function g
t = (0:N-1)';
g =@(omega) exp(1j*2*pi*omega*t/N)/sqrt(N);

% Calculate deltas
[~, zetas_scs, ~] = approximation_error(no_b_Deltas, b_i, Deltas, g);
max_zetas_scs = max(zetas_scs,[],2);

% Calculate regular BOMP errors
max_zetas_bomp_scs = zeros(size(max_zetas_scs));
for ii=1:length(c)
    max_zetas_bomp_scs(ii) = norm(g(b_i)-g(b_i+Deltas(ii)/2),2);
end

%% Plotting
h=figure(1);
semilogy(c, max_zetas_tde, 'b-x'); hold on;
semilogy(c, max_zetas_bomp_tde, 'b--o');
semilogy(c, max_zetas_scs, 'r-x');
semilogy(c, max_zetas_bomp_scs, 'r--o');
xlabel('c');
ylabel('$\zeta$','Interpreter','latex');
legend('Polar TDE', 'BOMP TDE', 'Polar FE', 'BOMP FE');
matlab2tikz('figurehandle',h, 'approximation_noise_analysis.tikz', 'height', '\figureheight', 'width', '\figurewidth', 'showInfo', false, 'checkforupdates', false);