%% PLOT_LAMBDA_NUM_EXP plots the contents of a numerical experiment file
%
% [ ] = PLOT_LAMBDA_NUM_EXP(FILENAME, VARIABLEOPT, SAVEPLOT, YSCALELOG)
%
% This function plots the contents in FILENAME.
%
% Input variables:
%   FILENAME    The filename for the file to use in plotting.
%   VARIABLEOPT Whether to plot with SNR and lambda on the x- and y-axis 
%                   (VARIABLEOPT=0) or with kappa and lambda on the x- and
%                   y-axis (VARIABLEOPT=1).
%   SAVEPLOT    Whether to save the plot as a tikz file (SAVEPLOT=1) or
%                   not (SAVEPLOT=0).
%   YSCALELOG   Whether to use a logarithmic scale on the y-axis
%                   (YSCALELOG=1) or not (YSCALELOG=0).
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function plot_lambda_num_exp(filename, variableopt, saveplot, yscalelog)
addpath('../misc');
results = load(filename);
bMSE_errors = results.bMSE_errors;
measSNRs = results.measSNRs; lambdas = results.lambdas; kappas = results.kappas;

% Averaging the results
noRuns = size(bMSE_errors, 5);
mean_bMSE_errors = squeeze(mean(bMSE_errors, 5));

% Plotting
if variableopt==0
    [X, Y] = meshgrid(measSNRs, lambdas);
    xlabeltext = 'SNR [dB]';
    ylabeltext = '$\lambda$';
elseif variableopt==1
    [X, Y] = meshgrid(kappas, lambdas);
    xlabeltext = '$\kappa$';
    ylabeltext = '$\lambda$';
end
clf();
usecs = 10^6;
surface(X, Y, log10(mean_bMSE_errors*usecs^2));
colormap(gray);
if yscalelog
    set(gca, 'yscale','log');
end
if saveplot
    title(strcat('Number of runs: ',int2str(noRuns)));
else
    title(strcat(filename,' - number of runs: ',int2str(noRuns)), 'Interpreter', 'none');
end
colorbar();
xlabel(xlabeltext,'Interpreter','latex');
ylabel(ylabeltext,'Interpreter','latex');
zlabel('b-MSE','Interpreter','latex');

if saveplot
    matlab2tikz('figurehandle', gcf(), strcat(filename,'.tikz'), 'height', '\figureheight', 'width', '\figurewidth', 'showInfo', false,'checkforupdates',false);
end
