%% APPROXIMATION_ERROR finds the approximation and reconstruction error for polar interpolation
%
% [ DELTAS, ZETAS, XAXIS ] = APPROXIMATION_ERROR(NO_B_DELTAS, B_I, DELTAS, G)
%
% This function finds the approximation and reconstruction error incurred
% by using polar interpolation on a given translation-invariant signal
% class.
%
% Input variables:
%   NO_B_DELTAS     The number of B_DELTA values to try.
%   B_I             The center translation parameter value to use.
%   DELTAS          The parameter sampling period.
%   G               The function for the translation-invariant signal.
%
% Output variables:
%   DELTAS          The approximation errors.
%   ZETAS           The reconstruction errors.
%   XAXIS           The translation parameters tested.
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [deltas, zetas, xaxis] = approximation_error(no_b_Deltas, b_i, Deltas, g)
    xaxis = linspace(-1, 1, no_b_Deltas);
    deltas = zeros(length(Deltas), no_b_Deltas);
    zetas = zeros(length(Deltas), no_b_Deltas);
    for ii=1:length(Deltas)
        Delta = Deltas(ii);
        b_Deltas = xaxis * Delta/2;
        for jj=1:no_b_Deltas
            b_Delta = b_Deltas(jj);
            gi = g(b_i);
            gb = g(b_i+b_Delta);
            gpDelta = g(b_i+Delta/2);
            gmDelta = g(b_i-Delta/2);
            r = norm(gi,2); 
            theta = acos(real(gi'*gpDelta)/(norm(gi,2)*norm(gpDelta,2)));
            rotMtx = [  1., r*cos(theta),    -r*sin(theta);
                1., r,                  0.;
                1., r*cos(theta),    r*sin(theta)];
            
            % Find delta
            theta_b_Delta = acos(real(gi'*gb)/(norm(gi,2)*norm(gb,2)));
            theta_Delta = acos(real(gi'*gpDelta)/(norm(gi,2)*norm(gpDelta,2)));
            deltas(ii,jj) = abs(2*abs(b_Delta)/Delta - theta_b_Delta/theta_Delta);
            
            % Find zeta
            mtx = rotMtx \ [transpose(gmDelta); transpose(gi); transpose(gpDelta)];
            c = transpose(mtx(1,:)); u = transpose(mtx(2,:)); v = transpose(mtx(3,:)); 
            gb_hat = c + r*cos(2*b_Delta*theta/Delta)*u + r*sin(2*b_Delta*theta/Delta)*v;
            zetas(ii,jj) = norm(gb-gb_hat,2);
        end    
    end
end