%% GEN_RDMATRIX generates a Random Demodulator sampling matrix of size MxN.
%
% [ RDMTX ] = GEN_RDMATRIX(M,N)
%
% GEN_RDMATRIX generates a Random Demodulator matrix [1] with M rows and
% N columns. All rows are orthogonal and normalized to have unit energy.
%
% Input variables:
%   M   The number of rows.
%   N   The number of columns.
%
% Output variables:
%   RDMTX   The Random Demodulator sampling matrix of size M-by-N.
% 
% References:
%   [1] J. A. Tropp et al., “Beyond Nyquist: Efficient sampling of sparse 
%       bandlimited signals,” IEEE Trans. Inf. Theory, vol. 56, no. 1, 
%       pp. 520–544, Jan. 2010.
% 
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [ RDmtx ] = genRDmtx( M, N )
RDVec = 2*(randi(2,1,N)-1)-1;
RDmtx = zeros(M,N);
MNratio = floor(N/M);
if MNratio < 1
    MNratio = 1;
end
idx2 = 0;
for kk = 1:M
    idx1 = idx2+1;
    if kk <= mod(N,M)
        idx2 = idx1+MNratio;
        colIdx = linspace(idx1, idx2, MNratio+1);
    else
        idx2 = idx1+MNratio-1;
        colIdx = linspace(idx1, idx2, MNratio);
    end
    RDmtx(kk,colIdx) = RDVec(colIdx)/norm(RDVec(colIdx),2);
end
end

