%% CCBP implements the Complex Continous Basis Pursuit algorithm.
%
% [F_EST, BS_EST] = CCBP(Y, A, OMEGA, SETTS, CCBP_VARS)
%
% This the implementation of the CCBP algorithm, based on the CBP algorithm
% from [1].
%
% Input variables:
%   Y           The measurements.
%   A           The measurement matrix.
%   OMEGA       The frequencies to generate a dictionary from.
%   SETTS       The known signal parameters.
%   CCBP_VARS   Specific CCBP variables.
%
% Output variables:
%   F_EST       The reconstructed signal.
%   BS_EST      The estimated translation parameters in the signal.
% 
% References:
%   [1] C. Ekanadham, D. Tranchina, and E. P. Simoncelli, "Recovery of
%   sparse translation-invariant signals with continuous basis pursuit",
%   IEEE Transactions on Signal Processing, vol. 59, no. 10, pp. 4735-4744,
%   Oct. 2011.
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [f_est, bs_est] = ccbp(y, A, Omega, setts, cbp_vars)
K=setts.K; Delta=setts.Delta; lambda=setts.lambda; 
eps = setts.eps; min_b=setts.min_b; max_b=setts.max_b;
J = length(Omega);

% Load CBP variables
C       = cbp_vars.C;
U       = cbp_vars.U;
V       = cbp_vars.V;
r       = cbp_vars.r;
theta   = cbp_vars.theta;
zeta    = cbp_vars.zeta;
eps = max([eps; 10^(-8)]);
tau = 2*(eps + zeta)*lambda;

D = [C 1j*C -C -1j*C U 1j*U -U -1j*U V 1j*V -V -1j*V];

d = 4*J;
Ai = {}; bi = {}; ci = {}; di = {}; ei = {};
for ii=1:d,
    tmp = zeros(3,3*d); tmp(1,ii) = 1; tmp(2,ii+d) = 1; tmp(3,ii+2*d) = 1;
    Ai{ii} = tmp;
    tmp = zeros(1,3*d); tmp(ii) = -r*cos(theta); tmp(ii+d) = 1;
    bi{ii} = tmp;
    tmp = zeros(1,3*d); tmp(ii) = 1;
    ci{ii} = tmp;
    tmp = zeros(1,3*d); tmp(ii) = r; tmp(ii+d) = -1;
    di{ii} = tmp;
end
for jj=1:J
    tmp = zeros(4,3*d); 
    tmp(1,jj)       = 1;    tmp(2,J+jj)     = 1j; 
    tmp(3,2*J+jj)   = -1;   tmp(4,3*J+jj)   = -1j;
    ei{jj} = tmp;
end
Aa = zeros(1,3*d);

cvx_quiet(1);
cvx_precision best;
cvx_begin
    variable x(3*d);
    variable t(J);
    minimize( pow_pos(norm(y-A*D*x),2) + tau*norm(t,1) );
    subject to
        for ii = 1:d
            % y(ii)^2 + z(ii)^2 <= x(ii)^2*r^2;
            norm(Ai{ii}*x,2) <= (ci{ii}*sqrt(r^2+1))*x;
            % x(ii)*r*cos(theta) <= y(ii) <= x(ii)*r;
            norm(Aa*x,2) <= bi{ii}*x;
            norm(Aa*x,2) <= di{ii}*x;
        end
        for jj = 1:J
            t(jj) >= norm(ei{jj}*x,2);
        end
cvx_end

alphap  = x(0*J+1:1*J) + 1j*x(1*J+1:2*J);   
alphan  = x(2*J+1:3*J) + 1j*x(3*J+1:4*J);
betap   = x(4*J+1:5*J) + 1j*x(5*J+1:6*J);   
betan   = x(6*J+1:7*J) + 1j*x(7*J+1:8*J);
gammap  = x(8*J+1:9*J) + 1j*x(9*J+1:10*J);   
gamman  = x(10*J+1:11*J) + 1j*x(11*J+1:12*J);

betap = betap.*alphap*r./sqrt(betap.^2 + gammap.^2);
betan = betan.*alphan*r./sqrt(betan.^2 + gamman.^2);
gammap = gammap.*alphap*r./sqrt(betap.^2 + gammap.^2);
gamman = gamman.*alphan*r./sqrt(betan.^2 + gamman.^2);

alpha = alphap-alphan; beta = betap-betan; gamma = gammap-gamman;

f_est = C*alpha + U*beta + V*gamma;

[~, idxs] = sort(abs(alpha),1,'descend');
run_bomp = 0;
excluded_atoms = [];
bs_est = zeros(K,1);
found = 0;
if sum(abs(alpha)) < 10^(-10)
    run_bomp = 1;
else    
    for jj = 1:J
        if sum(idxs(jj) == excluded_atoms) == 0
            beta_ii = beta(idxs(jj));
            gamma_ii = gamma(idxs(jj));
            bs_est(jj) = Omega(idxs(jj)) + atan(real(gamma_ii)/real(beta_ii))*Delta/(2*theta);
            if isnan(bs_est(jj))
                bs_est(jj) = 0;
            end
            if bs_est(jj) > max_b || bs_est(jj) < min_b
                run_bomp = 1;
            end
            if J > K
                if idxs(jj) == 1
                    exclude = [idxs(jj); idxs(jj)+1];
                elseif idxs(jj) == J
                    exclude = [idxs(jj)-1; idxs(jj)];
                else
                    exclude = [idxs(jj)-1; idxs(jj); idxs(jj)+1];
                end
                excluded_atoms = cat(1, excluded_atoms, exclude);
            end
            found = found + 1;
            if found == K
                break;
            end
        end
    end
end
if run_bomp
    [f_est, bs_est] = ibomp(y, A, setts, 'bomp');
else
    bs_est = bs_est(bs_est ~= 0);
end
end