%% GEN_CCBP_VARIABLES generates CCBP-specific variables
%
% [ CCBP_VARS ] = GEN_CCBP_VARIABLES(SETTS, OMEGA)
%
% This function generates a set of parameters that are needed in the CCBP 
% algorithm, but which it speeds up the algorithm to find beforehand.
%
% Input variables:
%   SETTS   Parameter set with experiment specific data.
%   OMEGA   Sampled parameter sequence.
%
% Output variables:
%   CCBP_VARS   The CCBP variables.
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [ ccbp_vars ] = gen_ccbp_variables(setts, Omega)
N=setts.N; g=setts.g; Delta=setts.Delta;
J = length(Omega);

% Make polar interpolation dictionaries, C, U and V
C = zeros(N,J);
U = zeros(N,J);
V = zeros(N,J);

no_b_Deltas = 100;
zetas = zeros(no_b_Deltas, J);
for jj = 1:J
    Funcs = zeros(3,N);
    Funcs(1,:) = g(Omega(jj)-Delta/2);
    Funcs(2,:) = g(Omega(jj));
    Funcs(3,:) = g(Omega(jj)+Delta/2);
    r = norm(Funcs(1,:),2);
    theta = acos(real(Funcs(1,:)*Funcs(2,:)'));
    rotMtx = [  1., r*cos(theta),    -r*sin(theta);
                1., r,                  0.;
                1., r*cos(theta),    r*sin(theta)];
    rotMtx = inv(rotMtx);
    C(:,jj) = rotMtx(1,:)*Funcs;
    U(:,jj) = rotMtx(2,:)*Funcs;
    V(:,jj) = rotMtx(3,:)*Funcs;
    [~, zetas(:,jj), ~] = approximation_error(no_b_Deltas, Omega(jj), Delta, g);
end
zeta = max(mean(zetas,2));

% Set output variables
ccbp_vars.C = C;
ccbp_vars.U = U;
ccbp_vars.V = V;
ccbp_vars.r = r;
ccbp_vars.theta = theta;
ccbp_vars.zeta = zeta;
end