%% IBOMP Interpolating Band-excluded Orthogonal Matching Pursuit algorithm
%
% [F_EST, BS_EST] = IBOMP(Y, A, SETTS, METHOD)
%
% The Interpolating Band-Excluded Orthogonal Matching Pursuit (IBOMP)
% algorithm uses Band-Excluded Orthogonal Matching Pursuit to find a rough
% estimate of the delay value, after which parabolic or polar interpolation
% may be used to improve on that estimate. Finally, CCBP may be used to
% further improve the estimate.
%
% Input variables:
%   Y       The measurements.
%   A       The measurement matrix.
%   SETTS   The numerical experiments variables.
%   METHOD  Which method to use: 'BOMP', 'PaIBOMP', PaIBOMP+CCBP' or 
%               'PoIBOMP'.
%
% Output variables:
%   F_EST   The reconstructed signal.
%   BS_EST  The estimated translation parameters.
% 
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [f_est, bs_est] = ibomp(y, A, setts, method)
    N=setts.N; K=setts.K; Delta=setts.Delta; eta=setts.eta; 
    g=setts.g; D=setts.D; mu=setts.mu; eps=setts.eps; xi=setts.xi;
    min_b=setts.min_b; max_b=setts.max_b;
    yr = y;
    P = size(D,2);
    B = zeros(N, K);
    as_est = zeros(K,1);
    bs_est = zeros(K,1);
    S = [];
    for ii = 1:K
        proxy = abs((A*D)'*yr);
        BE = band_exclusion(S, mu, eta);
        proxy(BE==1) = 0;
        [~, idx] = max(proxy);
        
        % Fix border-cases due to wrap-around in dictionary
        if (idx-1)*Delta > max_b
            idx = 1;
        end
        
        % Use chosen method to make estimate
        if strcmpi(method, 'BOMP')
            bs_est(ii) = (idx-1)*Delta;
        elseif strcmpi(method, 'PaIBOMP') || strcmpi(method, 'PaIBOMP+CCBP')
            bs_est(ii) = parabolic(idx, proxy, setts);
        elseif strcmpi(method, 'PoIBOMP')
            bs_est(ii) = polar(idx, yr, A, proxy, setts);
        else
            error('Unknown interpolation method!');
        end
        B(:,ii) = g(bs_est(ii));
        as_est(1:ii) = (A*B(:,1:ii)) \ y;
        comp_sig = A*B(:,1:ii)*as_est(1:ii);
        yr = y-comp_sig;
        S = cat(2, S, idx);
        if norm(yr,2) <= 2*eps
            break;
        end
    end
    
    % Use CCBP to improve estimate
    if strcmpi(method, 'PaIBOMP+CCBP')
        Omega = [];
        for jj=1:K
            Omega = union(Omega, (S(jj)-xi):(S(jj)+xi));
        end
        Omega = (Omega(Omega>=1 & Omega<=P)-1)*Delta; % Remove invalid border-cases
        cbp_vars = gen_ccbp_variables(setts, Omega);
        [f_est, bs_est] = ccbp(y, A, Omega, setts, cbp_vars);
        for jj=1:length(bs_est)
            if bs_est(jj) == 0
                bs_est(jj) = (S(jj)-1)*Delta;
            end
        end
    else
        f_est = B(:,1:ii)*as_est(1:ii);
    end
    
    % Should something have gone wrong, use BOMP as backup
    if sum((bs_est > max_b) + (bs_est < min_b)) && ~strcmp(method,'bomp')
        [f_est, bs_est] = ibomp(y, A, setts, 'bomp');
    end    
end