%% L1SYNTHESIS implements the $\ell_1$-synthesis algorithm.
%
% [ X_HAT ] = L1SYNTHESIS(Y, A, D, EPSILON)
%
% This function implements the $\ell_1$-synthesis algorithm from [1].
%
% Input variables:
%   Y               The measurements.
%   A               The measurement matrix.
%   D               The dictionary matrix.
%   EPSILON         The power of the noise in the signal. If EPSILON = 0,
%                   the Basis Pursuit algorithm is used, otherwise the 
%                   Basis Pursuit Denoising algorithm is used.
%
% Output variables:
%   X_HAT           The reconstructed coefficients for the dictionary matrix.
% 
% References:
%   [1] E. J. Candes, Y. C. Eldar, D. Needell, and P. Randall, "Compressed
%       sensing with coherent and redundant dictionaries", Applied and
%       Computational Harmonic Analysis, vol. 31, no. 1, pp. 59-73, 2011.
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function x_hat = l1synthesis(y, A, D, epsilon)

p = size(D,2);

if epsilon == 0
    % Synthesis
    cvx_quiet(1);
    cvx_precision best;
    cvx_begin
        variable x_hat(p) complex;
        minimize( norm(x_hat,1) );
        subject to 
            y == A*D*x_hat;
    cvx_end
else
    if epsilon < 10^(-8)
        epsilon = 10^(-8);
    end

    % Synthesis
    cvx_quiet(1);
    cvx_precision best;
    cvx_begin
        variable x_hat(p) complex;
        minimize( norm(x_hat,1) );
        subject to 
            norm(A*D*x_hat - y,2) <= epsilon*norm(y,2);
    cvx_end
end
if sum(isnan(x_hat)) > 0
    x_hat = zeros(p,1);
end