%% PARABOLIC is the implementation of the parabolic interpolation.
%
% [ B_EST ] = PARABOLIC(IDX, PROXY, SETTS)
%
% The parabolic interpolation is one method to improve on the delay
% estimate from the found atoms by the BOMP algorithm. The implementation
% we employ here is based on the Direct Correlator from [1].
%
% Input variables:
%   IDX     The index in the proxy corresponding to our rough
%               estimate of the delay.
%   PROXY   The proxy for the residual.
%   SETTS   The numerical experiment settings.
%
% Output variables:
%   B_EST   The estimated translation parameter.
%
% References:
%   [1] G. Jacovitti and G. Scarano, "Discrete time techniques for time
%   delay estimation," IEEE Trans. Signal Process., vol. 41, no. 2, pp.
%   525-533, Feb. 1993.
% 
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [ b_est ] = parabolic(idx, proxy, setts)
    delta = setts.Delta;
    % Find center and parabola points
    Rx = zeros(1, 3);
    if idx==1
        Rx(1) = proxy(idx);
        Rx(2) = proxy(idx+1);
        Rx(3) = proxy(idx+2);
    elseif idx==length(proxy)
        Rx(1) = proxy(idx-2);
        Rx(2) = proxy(idx-1);
        Rx(3) = proxy(idx);
    else
        Rx(1) = proxy(idx-1);
        Rx(2) = proxy(idx);
        Rx(3) = proxy(idx+1);
    end
    if (Rx(1)-2*Rx(2)+Rx(3)) == 0
        b_est = (idx-1)*delta; % Rx entries are identical, so the interpolation cannot be trusted
        return
    end
    tau_center = delta*(idx-1);
    b_est = -delta/2 * (Rx(3) - Rx(1))/(Rx(3)-2*Rx(2)+Rx(1)) + tau_center;
    return
end

