%% POLAR is the implementation of the polar interpolation.
%
% [B_EST] = POLAR(IDX, YR, A, PROXY, SETTS)
%
% The polar interpolation is one method to improve on the delay
% estimate from the found atoms by the BOMP algorithm. The implementation
% we employ here is based on the polar interpolator from [1].
%
% Input variables:
%   IDX     The index in the proxy corresponding to our rough
%               estimate of the delay.
%   YR      The residual of the received signal.
%   A       The measurement matrix.
%   PROXY   The proxy for the residual.
%   SETTS   The numerical experiment settings.
%
% Output variables:
%   B_EST   The estimated translation parameter.
%
% References:
%   [1] C. Ekanadham, D. Tranchina, and E.P. Simoncelli, "Re- covery of
%   sparse translation-invariant signals with con- tinuous basis pursuit",
%   IEEE Trans. Signal Process., vol. 59, no. 10, pp. 4735-4744, Oct. 2011.
% 
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [ b_est ] = polar(idx, yr, A, proxy, setts)
    N=setts.N; Delta=setts.Delta; g=setts.g; D=setts.D;
    M = size(A,1);
    f_m = g((idx-1)*Delta-Delta/2);
    f_0 = D(:,idx);
    f_p = g((idx-1)*Delta+Delta/2);
    r = norm(f_0,2);
    theta = acos(real(f_0'*f_p)/r^2);
    
    rotMtx = [   1., r*cos(theta),    -r*sin(theta);
            1., r,                  0.;
            1., r*cos(theta),    r*sin(theta)];
    rotMtx = inv(rotMtx);

    Funcs = zeros(3,N);
    Funcs(1,:) = f_m;
    Funcs(2,:) = f_0;
    Funcs(3,:) = f_p;

    B = zeros(N,3);
    B(:,1) = rotMtx(1,:)*Funcs;
    B(:,2) = rotMtx(2,:)*Funcs;
    B(:,3) = rotMtx(3,:)*Funcs;

    tau_0 = (idx-1)*Delta;
    if rank(A*B) < 3
        b_est = tau_0;
    elseif M/N < 0.975 % This value is found experimentally
        coef = (A*B) \ yr;
        if coef(2)==0
            ang = 0;
        else
            ang = atan(coef(3)/coef(2));
        end
        ang = real(ang) + imag(ang); 
        delta_tau = ang*Delta/(2*theta);
        b_est = tau_0 + delta_tau;
    else
       coef = (A*B) \ yr;
       if real(coef(2)/coef(1))/r > 1 % Handle special case where acos returns complex result
           tau1 = 0;
       else
           tau1 = acos(real(coef(2)/coef(1))/r) * Delta/(2*theta);
       end
       tau2 = asin(real(coef(3)/coef(1))/r) * Delta/(2*theta);
       delta_tau = 0.5*(abs(tau1)+abs(tau2));
       buf_proxy = zeros(size(proxy,1)+2,1);
       buf_proxy(2:length(proxy)+1) = proxy;
       if abs(buf_proxy(idx)) < abs(buf_proxy(idx+2))
           b_est = tau_0 + delta_tau;
       else
           b_est = tau_0 - delta_tau;
       end
    end
end

