%% TDE_MUSIC runs the TDE MUSIC algorithm to estimate delays.
%
% [F_EST, B_EST] = TDE_MUSIC(Y, A, SETTS)
%
% TDE_MUSIC reconstructs the Nyquist-rate sampled signal and estimates the
% time delay of the function, based on a compressive sensing measurement
% vector and knowledge of the sparsity, K, and the noise variance, EPS.
%
% Input variables:
%   Y       The measurements vector.
%   A       The measurement matrix.
%   SETTS   The numerical experiment settings.
%
% Output variables:
%   F_EST   The reconstructed signal.
%   B_EST   The estimated translation parameters.
% 
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [f_est, b_est] = tde_music(y, A, setts)
N=setts.N; K=setts.K; dt=setts.dt; D=setts.D;

% Reconstruct original signal
x_hat = l1synthesis(y, A, D, setts.eps);
f_est = D*x_hat;

% Reconstruct signal and estimate delays
S = fft(D(:,1));
H = diag(S) \ fft(f_est);
omega = N-musicapprox(H, K);
b_est = omega*dt;
end