%% PLOT_NUM_EXP plots the contents of a numerical experiment file
%
% [ ] = PLOT_NUM_EXP(FILENAME, SWEEPPARAM, LEGENDPOS, TITLEPOS, SAVEPLOT, 
%           STDLINES)
%
% This function plots the contents in FILENAME.
%
% Input variables:
%   FILENAME    The filename for the file to use in plotting.
%   SWEEPPARAM  Either 'measSNR', 'kappa' or 'sigSNR'.
%   LEGENDPOS   Which plot [1-3] to put a legend in. Specify as a number or
%                   a vector.
%   TITLEPOS    Which plot [1-3] to put a title on. Specify as a number or
%                   a vector.
%   SAVEPLOT    Whether to save the figure as a tikz file (SAVEPLOT=1) or
%                   not (SAVEPLOT=0).
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function plot_num_exp(filename, sweepparam, legendpos, titlepos, saveplot, stdlines)
addpath('../misc');
results = load(filename);
f_errors = results.f_errors;
bMSE_errors = results.bMSE_errors;
comp_times = results.comp_times;
methods = results.methods; kappas = results.kappas; 
measSNRs = results.measSNRs; sigSNRs = results.sigSNRs;

% Averaging the results
noRuns = size(f_errors,5);
mean_f_errors = squeeze(mean(f_errors, 5));
std_f_errors =  squeeze(std(f_errors, 0, 5));
mean_betaMSE_errors = squeeze(mean(bMSE_errors, 5));
std_betaMSE_errors =  squeeze(std(bMSE_errors, 0, 5));
mean_comp_times = squeeze(mean(comp_times, 5));
std_comp_times =  squeeze(std(comp_times, 0, 5));

% Plotting
ii = 1;
if sum(strcmp(methods, 'BOMP'))
    colors{ii} = [0    0    0    ]; markers{ii} = 'x'; ii = ii+1;
end
if sum(strcmp(methods, 'PaIBOMP'))
    colors{ii} = [0    0    1    ]; markers{ii} = 'o'; ii = ii+1;
end
if sum(strcmp(methods, 'TDE-MUSIC'))
    colors{ii} = [0    0.75 0.25 ]; markers{ii} = '<'; ii = ii+1;
end
if sum(strcmp(methods, 'CCBP'))
    colors{ii} = [0.5  1    0    ]; markers{ii} = 'p'; ii = ii+1;
end
if sum(strcmp(methods, 'PaIBOMP+CCBP'))
    colors{ii} = [0.75 0    0.25 ]; markers{ii} = '^'; ii = ii+1;
end
if sum(strcmp(methods, 'PoIBOMP'))
    colors{ii} = [1    0.5  0.5  ]; markers{ii} = 'v'; 
end

if strcmp(sweepparam, 'measSNR')
    xaxis = measSNRs;
    xaxislabel = 'SNR [dB]';
elseif strcmp(sweepparam, 'kappa')
    xaxis = kappas;
    xaxislabel = '\kappa';
elseif strcmp(sweepparam, 'sigSNR')
    xaxis = sigSNRs;
    xaxislabel = 'SNR [dB]';
end

% f_errors
h=figure(1); clf;
for ii = 1:length(methods)
    semilogy(xaxis, mean_f_errors(ii,:), 'Color', colors{ii}, 'Marker', markers{ii}); hold on
    if stdlines
        semilogy(xaxis, mean_f_errors(ii,:)+std_f_errors(ii,:), 'Color', colors{ii}, 'LineStyle', ':');
        %semilogy(xaxis, mean_f_errors(ii,:)-std_f_errors(ii,:), 'Color', colors{ii}, 'LineStyle', ':');
    end
end
xlabel(xaxislabel);
ylabel('Function norm-2 error');
ylim([10^(-3) 10^3]);
if sum(titlepos == 1)
    title(strcat(filename,' - number of runs: ',int2str(noRuns)), 'Interpreter', 'none');
end
if sum(legendpos == 1)
    legend(methods, 'Location', 'NorthOutside','Orientation','horizontal');
end
if saveplot
    matlab2tikz('figurehandle',h, strcat(filename,'_ferrors.tikz'), 'height', '\figureheight', 'width', '\figurewidth', 'showInfo', false,'checkforupdates',false);
end

% betaMSE
usecs = 10^6;
h=figure(2); clf;
for ii = 1:length(methods)
    semilogy(xaxis, mean_betaMSE_errors(ii,:)*usecs^2, 'Color', colors{ii}, 'Marker', markers{ii}); hold on
    if stdlines
        semilogy(xaxis, (mean_betaMSE_errors(ii,:)+std_betaMSE_errors(ii,:))*usecs^2, 'Color', colors{ii}, 'LineStyle', ':');
        %semilogy(xaxis, (mean_betaMSE_errors(ii,:)-std_betaMSE_errors(ii,:))*usecs^2, 'Color', colors{ii}, 'LineStyle', ':');
    end
end
xlabel(xaxislabel);
ylabel('\vb-MSE [\mu{}s^2]');%, 'Interpreter','none');
ylim([10^(-9) 10^(1)]);
if sum(titlepos == 2)
    title(strcat(filename,' - number of runs: ',int2str(noRuns)), 'Interpreter', 'none');
end
if sum(legendpos == 2)
    legend(methods, 'Location', 'NorthOutside','Orientation','horizontal');
end
if saveplot
    matlab2tikz('figurehandle',h, strcat(filename,'_betamse.tikz'), 'height', '\figureheight', 'width', '\figurewidth', 'showInfo', false,'checkforupdates',false);
end

% Computation time
h=figure(3); clf;
for ii = 1:length(methods)
    semilogy(xaxis, mean_comp_times(ii,:), 'Color', colors{ii}, 'Marker', markers{ii}); hold on
    if stdlines
        semilogy(xaxis, mean_comp_times(ii,:)+std_comp_times(ii,:), 'Color', colors{ii}, 'LineStyle', ':');
        %semilogy(xaxis, mean_comp_times(ii,:)-std_comp_times(ii,:), 'Color', colors{ii}, 'LineStyle', ':');
    end
end
xlabel(xaxislabel);
ylabel('Average computation time [s]');
ylim([10^(-2) 10^4]);
if sum(titlepos == 3)
    title(strcat(filename,' - number of runs: ',int2str(noRuns)), 'Interpreter', 'none');
end
if sum(legendpos == 3)
    legend(methods, 'Location', 'NorthOutside','Orientation','horizontal');
end
if saveplot
    matlab2tikz('figurehandle',h, strcat(filename,'_comptimes.tikz'), 'height', '\figureheight', 'width', '\figurewidth', 'showInfo', false,'checkforupdates',false);
end
end
