%% SIMPLE_TDE_EXAMPLE is an example script for a single TDE problem
% 
% This is an example script used for solving one specific TDE problem. 
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
clear; clc;
addpath('signal_generators','recon_algorithms','misc');

% Set random seed
seed = 1;
if seed~=0
    s = RandStream('mcg16807','Seed',seed);
    RandStream.setGlobalStream(s);
end

% Experiment variables
kappa = 1;
measSNR = 1000;
sigSNR = 1000;

% Experiment parameters
N = 500;
M = round(N*kappa);
K = 3;
b_sep = 10^(-6);
eta = 0;
xi = 0;
lambdas = 10^0;

% Function g
f_s = 50*10^6;
dt = 1/f_s;
Delta = dt;
t = (0:(N-1))'*dt;
f_0 = 10^6; delta_f = 40*f_0; T = 10^(-6);
g =@(tau) chirp(t-tau, f_0, delta_f, T);
min_b = 0;
max_b = t(N)-T;

% Dictionary
D = zeros(N);
Omega = (0:Delta:((N-1)*Delta-T));
D(:,1) = g(Omega(1));
for ii = 2:N
    D(:,ii) = circshift(D(:,ii-1),1);
end
mu = abs(D'*D);

% Signal parameters
min_a = 1;
max_a = 10;
bs = gen_bs(min_b, max_b, K, b_sep);
as = gen_as(min_a, max_a, K);
display(bs);
display(as);

% Generate signal
f = zeros(N,1);
for ii = 1:K
    f = f + as(ii)*g(bs(ii));
end

% Generate measurements
A = genRDmtx(M, N);

% Add signal noise
f_pwr = mean(abs(f).^2);
sig_noise_pwr = f_pwr / 10^(sigSNR/10);
sig_noise = sqrt(sig_noise_pwr/2) * (randn(size(f)) + 1i*randn(size(f)));
f_noisy = f + sig_noise;
sig_eps = norm(sig_noise,2)/norm(f,2);

% Sample signal
y = A*f_noisy;

% Add measurement noise
y_pwr = mean(abs(y).^2);
meas_noise_pwr = y_pwr / 10^(measSNR/10);
meas_noise = sqrt(meas_noise_pwr/2) * (randn(size(y)) + 1i*randn(size(y)));
meas_eps = norm(meas_noise,2)/norm(y,2);
y = y + meas_noise;

% Reconstruct signal and estimate delay
setts.N     = N;
setts.K  = K;
setts.Delta = Delta;
setts.g     = g;
setts.D     = D;
setts.mu    = mu;
setts.eps   = eps/norm(y,2);
setts.eta   = eta;
setts.xi    = xi;
setts.min_b = min_b;
setts.max_b = max_b;
setts.b_sep = b_sep;
setts.min_a = min_a;
setts.max_a = max_a;
setts.dt    = dt;
ii = 1;
methods{ii} = 'BOMP';           ii = ii+1;
methods{ii} = 'PaIBOMP';        ii = ii+1;
methods{ii} = 'TDE-MUSIC';      ii = ii+1;
methods{ii} = 'CCBP';           ii = ii+1;
methods{ii} = 'PaIBOMP+CCBP';   ii = ii+1;
methods{ii} = 'PoIBOMP';                
noMethods = length(methods);

if sum(strcmp(methods, 'CCBP'))
    ccbp_vars = gen_ccbp_variables(setts, Omega);
    setts.ccbp_vars = ccbp_vars;
end

f_errors = zeros(noMethods,1);
bMSE_errors = zeros(noMethods,1);
comp_times = zeros(noMethods,1);

ii = 1;
% BOMP
if sum(strcmp(methods, 'BOMP'))
    tic;
    [f_est_bomp, bs_est_bomp] = ibomp(y, A, setts, 'BOMP');
    comp_times(ii) = toc;
    f_errors(ii) = norm(f_est_bomp-f,2)^2;
    bMSE_errors(ii) = b_mse_err(bs, bs_est_bomp);
    display(bs_est_bomp);
    ii = ii+1;
end

% PaIBOMP
if sum(strcmp(methods, 'PaIBOMP'))
    tic;
    [f_est_pabomp, bs_est_pabomp] = ibomp(y, A, setts, 'PaIBOMP');
    comp_times(ii) = toc;
    f_errors(ii) = norm(f_est_pabomp-f,2)^2;
    bMSE_errors(ii) = b_mse_err(bs, bs_est_pabomp);
    display(bs_est_pabomp);
    ii = ii+1;
end

% TDE-MUSIC
if sum(strcmp(methods, 'TDE-MUSIC'))
    tic;
    [f_est_tdemusic, bs_est_tdemusic] = tde_music(y, A, setts);
    comp_times(ii) = toc;
    f_errors(ii) = norm(f_est_tdemusic-f,2)^2;
    bMSE_errors(ii) = b_mse_err(bs, bs_est_tdemusic);
    display(bs_est_tdemusic);
    ii = ii+1;
end

% CCBP
if sum(strcmp(methods, 'CCBP'))
    for nn = 1:length(lambdas)
        setts.lambda = lambdas(nn);
        tic;
        [f_est_ccbp, bs_est_ccbp] = ccbp(y, A, Omega, setts, ccbp_vars);
        comp_times(ii) = toc;
        f_errors(ii) = norm(f_est_ccbp-f,2)^2;
        bMSE_errors(ii) = b_mse_err(bs, bs_est_ccbp);
        display(bs_est_ccbp);
        ii = ii+1;
    end
end

% BISP
if sum(strcmp(methods, 'PaIBOMP+CCBP'))
    for nn = 1:length(lambdas)
        setts.lambda = lambdas(nn);
        tic;
        [f_est_paibompccbp, bs_est_paibompccbp] = ibomp(y, A, setts, 'PaIBOMP+CCBP');
        comp_times(ii) = toc;
        f_errors(ii) = norm(f_est_paibompccbp-f,2)^2;
        bMSE_errors(ii) = b_mse_err(bs, bs_est_paibompccbp);
        display(bs_est_paibompccbp);
        ii = ii+1;
    end
end

% PoIBOMP
if sum(strcmp(methods, 'PoIBOMP'))
    tic;
    [f_est_poibomp, bs_est_poibomp] = ibomp(y, A, setts, 'PoIBOMP');
    comp_times(ii) = toc;
    f_errors(ii) = norm(f_est_poibomp-f,2)^2;
    bMSE_errors(ii) = b_mse_err(bs, bs_est_poibomp);
    display(bs_est_poibomp);
    ii = ii+1;
end

% Plot error signal
subplot('211');
bar(bMSE_errors);
set(gca,'XTickLabel', methods);
set(gca, 'yscale','log')
ylabel('b-MSE');
subplot('212');
bar(comp_times);
set(gca,'XTickLabel', methods)
ylabel('Average computation time [s]');