%% STICS_NUMEXP runs sparse translation-invariant signals experiments
%
% [F_ERRORS, BMSE_ERRORS, COMP_TIMES ] = STICS_NUMEXP(METHODS, LAMBDAS, 
%   KAPPAS, MEASSNRS, SIGSNRS, K, NORUNS, SETTS)
%
% This function runs NORUNS numerical experiments with sparse translation-
% invariant signals. In each run a signal is generated and noise is added 
% according to MEASSNRS and SIGSNRS. Then the translation parameter and
% signal reconstruction is estimated using the algorithms in METHODS. The 
% error of the results are output together with the computation time for 
% each algorithm.
%
% Input variables:
%   METHODS     Which algorithms to use: 'BOMP', 'PaIBOMP', 'TDE-MUSIC',
%                   'CCBP', 'PaIBOMP+CCBP' and/or 'PoIBOMP'.
%   LAMBDAS     The sparsity/fidelity factors to use in the experiments.
%   KAPPAS      The subsampling ratios to use in the experiments.
%   MEASSNRS    The SNR levels of the measurement noise to apply in the
%                   experiment.
%   SIGSNRS     The SNR levels of the signal noise to apply in the
%                   experiment.
%   K           The number of pulses in a signal.
%   NORUNS      How many experiments each call of STICS_NUMEXP shall 
%                   generate.
%   SETTS       A settings structure containing all experiment specific
%                   variables.
%
% Output variables:
%   F_ERRORS        The norm-2 error of the reconstructed signal.
%   BMSE_ERRORS     The MSE for the translation parameters
%   COMP_TIMES      The computation time for each algorithm
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function [f_errors, bMSE_errors, comp_times] = stics_numexp(methods, lambdas, kappas, measSNRs, sigSNRs, K, noRuns, setts)
min_b=setts.min_b; max_b=setts.max_b; b_sep=setts.b_sep; Omega = setts.Omega;
min_a=setts.min_a; max_a=setts.max_a; N=setts.N; g = setts.g;

noMethods = length(methods);
f_errors    = zeros(noMethods, length(kappas), length(measSNRs), length(sigSNRs), noRuns);
bMSE_errors = zeros(noMethods, length(kappas), length(measSNRs), length(sigSNRs), noRuns);
comp_times  = zeros(noMethods, length(kappas), length(measSNRs), length(sigSNRs), noRuns);

fprintf(1,'##############\n');
fprintf(1, 'Progress: %3d%%', 0);
for oo = 1:length(sigSNRs)
    sigSNR = sigSNRs(oo);
    for ii = 1:length(measSNRs)
        measSNR = measSNRs(ii);
        
        for jj = 1:length(kappas)
            kappa = kappas(jj);
            M = round(N*kappa);
            
            for mm = 1:noRuns
                % Generate signal
                f = zeros(N,1);
                bs = gen_bs(min_b, max_b, K, b_sep);
                as = gen_as(min_a, max_a, K);
                for kk = 1:K
                    f = f + as(kk)*g(bs(kk));
                end
                
                % Generate measurement matrix
                A = genRDmtx(M, N);
                
                % Add signal noise
                f_pwr = mean(abs(f).^2);
                sig_noise_pwr = f_pwr / 10^(sigSNR/10);
                sig_noise = sqrt(sig_noise_pwr/2) * (randn(size(f)) + 1i*randn(size(f)));
                f_noisy = f + sig_noise;
                sig_eps = norm(sig_noise,2)/norm(f,2);
                
                % Sample signal
                y = A*f_noisy;
                
                % Add measurement noise
                y_pwr = mean(abs(y).^2);
                meas_noise_pwr = y_pwr / 10^(measSNR/10);
                meas_noise = sqrt(meas_noise_pwr/2) * (randn(size(y)) + 1i*randn(size(y)));
                meas_eps = norm(meas_noise,2)/norm(y,2);
                y = y + meas_noise;
                                
                setts.eps = sig_eps + meas_eps;
                
                %%%% Reconstruct signal and estimate delay %%%%
                
                ll = 1;
                % BOMP
                if sum(strcmp(methods, 'BOMP'))
                    tic;
                    [f_est_bomp, bs_est_bomp] = ibomp(y, A, setts, 'BOMP');
                    comp_times(ll, jj, ii, oo, mm) = toc;
                    f_errors(ll, jj, ii, oo, mm) = norm(f_est_bomp-f,2)^2;
                    bMSE_errors(ll, jj, ii, oo, mm) = b_mse_err(bs, bs_est_bomp);
                    ll = ll+1;
                end
                
                % PaIBOMP
                if sum(strcmp(methods, 'PaIBOMP'))
                    tic;
                    [f_est_paibomp, bs_est_paibomp] = ibomp(y, A, setts, 'PaIBOMP');
                    comp_times(ll, jj, ii, oo, mm) = toc;
                    f_errors(ll, jj, ii, oo, mm) = norm(f_est_paibomp-f,2)^2;
                    bMSE_errors(ll, jj, ii, oo, mm) = b_mse_err(bs, bs_est_paibomp);
                    ll = ll+1;
                end
                
                % TDE-MUSIC
                if sum(strcmp(methods, 'TDE-MUSIC'))
                    tic;
                    [f_est_tdemusic, bs_est_tdemusic] = tde_music(y, A, setts);
                    comp_times(ll, jj, ii, oo, mm) = toc;
                    f_errors(ll, jj, ii, oo, mm) = norm(f_est_tdemusic-f,2)^2;
                    bMSE_errors(ll, jj, ii, oo, mm) = b_mse_err(bs, bs_est_tdemusic);
                    ll = ll+1;
                end
                
                % CCBP
                if sum(strcmp(methods, 'CCBP'))
                    for nn = 1:length(lambdas)
                        setts.lambda = lambdas(nn);
                        tic;
                        [f_est_ccbp, bs_est_ccbp] = ccbp(y, A, Omega, setts, setts.ccbp_vars);
                        comp_times(ll, jj, ii, oo, mm) = toc;
                        f_errors(ll, jj, ii, oo, mm) = norm(f_est_ccbp-f,2)^2;
                        bMSE_errors(ll, jj, ii, oo, mm) = b_mse_err(bs, bs_est_ccbp);
                        ll = ll+1;
                    end
                end
                
                % PaIBOMP+CCBP
                if sum(strcmp(methods, 'PaIBOMP+CCBP'))
                    for nn = 1:length(lambdas)
                        setts.lambda = lambdas(nn);
                        tic;
                        [f_est_paibompccbp, bs_est_paibompccbp] = ibomp(y, A, setts, 'PaIBOMP+CCBP');
                        comp_times(ll, jj, ii, oo, mm) = toc;
                        f_errors(ll, jj, ii, oo, mm) = norm(f_est_paibompccbp-f,2)^2;
                        bMSE_errors(ll, jj, ii, oo, mm) = b_mse_err(bs, bs_est_paibompccbp);
                        ll = ll+1;
                    end
                end
                
                % PoIBOMP
                if sum(strcmp(methods, 'PoIBOMP'))
                    tic;
                    [f_est_poibomp, bs_est_poibomp] = ibomp(y, A, setts, 'PoIBOMP');
                    comp_times(ll, jj, ii, oo, mm) = toc;
                    f_errors(ll, jj, ii, oo, mm) = norm(f_est_poibomp-f,2)^2;
                    bMSE_errors(ll, jj, ii, oo, mm) = b_mse_err(bs, bs_est_poibomp);
                end
                
                % Status
                status = round(100*(mm+(jj-1)*noRuns+(ii-1)*noRuns*length(kappas)+(oo-1)*noRuns*length(kappas)*length(measSNRs))/(noRuns*length(kappas)*length(measSNRs)*length(sigSNRs)));
                fprintf(1, '\b\b\b\b%3d%%', status);
            end
        end
    end
end
fprintf(1, '\n');
end