%% TDE_MAIN controls a TDE numerical experiment
%
% [ ] = TDE_MAIN(METHODS, N, K, LAMBDAS, KAPPAS, MEASSNRS, 
%   SIGSNRS, B_SEP, ETA, NORUNS, FILENAME)
%
% This function sets up the variables specific for a TDE translation-invariant
% problem and runs the while loop conducting experiments and saving the
% results to a file on the disk.
%
% Input variables:
%   METHODS     Which algorithms to use: 'BOMP', 'PaIBOMP', 'TDE-MUSIC',
%                   'CCBP', 'PaIBOMP+CCBP' and/or 'PoIBOMP'.
%   N           The size of the TDE signal
%   K           The number of pulses in a signal.
%   LAMBDAS     The sparsity/fidelity factors to use in the experiments.
%   KAPPAS      The subsampling ratios to use in the experiments.
%   MEASSNRS    The SNR levels of the measurement noise to apply in the
%                   experiment.
%   SIGSNRS     The SNR levels of the signal noise to apply in the
%                   experiment.
%   B_SEP       The minimum separation between pulses.
%   ETA         The band-exclusion parameter
%   NORUNS      How many experiments each call of STICS_NUMEXP shall 
%                   generate.
%   FILENAME    The filename to use to save the results to.
%
% Code implemented by: Karsten Fyhn
% Contact e-mail: kfn@es.aau.dk
%
% Copyright 2013 Karsten Fyhn
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
%   http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distribued on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and
% limitations under the License.
function tde_main(methods, N, K, lambdas, kappas, measSNRs, sigSNRs, b_sep, eta, noRuns, filename)
% Experiment parameters
xi = 0;

% Signal parameters
min_a = 1;
max_a = 10;

% Function g
f_s = 50*10^6;
dt = 1/f_s;
Delta = dt;
t = (0:(N-1))'*dt;
f_0 = 10^6; delta_f = 40*f_0; T = 10^(-6);
g =@(tau) chirp(t-tau, f_0, delta_f, T);
min_b = 0;
max_b = t(N)-T;

% Dictionary
D = zeros(N);
Omega = (0:Delta:((N-1)*Delta-T));
D(:,1) = g(Omega(1));
for ii = 2:N
    D(:,ii) = circshift(D(:,ii-1),1);
end
mu = abs(D'*D);

% Numerical experiment
setts.N         = N;
setts.K         = K;
setts.Delta     = Delta;
setts.g         = g;
setts.D         = D;
setts.Omega     = Omega;
setts.mu        = mu;
setts.eta       = eta;
setts.xi        = xi;
setts.min_b     = min_b;
setts.max_b     = max_b;
setts.b_sep     = b_sep;
setts.min_a     = min_a;
setts.max_a     = max_a;
setts.dt        = dt;

if sum(strcmp(methods, 'CCBP'))
    ccbp_vars = gen_ccbp_variables(setts, Omega);
    setts.ccbp_vars = ccbp_vars;
end

% Main simulation
seeds = []; f_errors = []; bMSE_errors = []; comp_times = [];
ii = 1;
while 1
    % Set random seed
    seed = mod(now*rand, int16(10^6));
    s = RandStream('mcg16807','Seed',seed);
    RandStream.setGlobalStream(s);
    fprintf('\n\nIteration: %d\nSeed is: %d\n', ii, seed);
    [f_errors_tmp, bMSE_errors_tmp, comp_times_tmp] = stics_numexp(methods, lambdas, kappas, measSNRs, sigSNRs, K, noRuns, setts);
    seeds = cat(1, seeds, seed);
    f_errors = cat(5, f_errors, f_errors_tmp);
    bMSE_errors = cat(5, bMSE_errors, bMSE_errors_tmp);
    comp_times = cat(5, comp_times, comp_times_tmp);
    save(strcat('tmp/',filename), 'seeds', 'methods', 'f_errors', 'bMSE_errors', 'comp_times', 'lambdas', 'kappas', 'measSNRs', 'sigSNRs', 'K');
    ii = ii+1;
end



